@@ -1,9 +1,9 @@
-/** \file database.h
- * \brief API for working with Xapian databases
+/** @file database.h
+ * @brief API for working with Xapian databases
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011,2012,2013,2014,2015,2016,2017,2019 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -25,11 +25,20 @@
 #ifndef XAPIAN_INCLUDED_DATABASE_H
 #define XAPIAN_INCLUDED_DATABASE_H
 
+#if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
+# error Never use <xapian/database.h> directly; include <xapian.h> instead.
+#endif
+
+#include <iosfwd>
 #include <string>
+#ifdef XAPIAN_MOVE_SEMANTICS
+# include <utility>
+#endif
 #include <vector>
 
-#include <xapian/base.h>
-#include <xapian/document.h>
+#include <xapian/attributes.h>
+#include <xapian/deprecated.h>
+#include <xapian/intrusive_ptr.h>
 #include <xapian/types.h>
 #include <xapian/positioniterator.h>
 #include <xapian/postingiterator.h>
@@ -39,6 +48,9 @@
 
 namespace Xapian {
 
+class Compactor;
+class Document;
+
 /** This class is used to access a database, or a group of databases.
  *
  *  For searching, this class is used in conjunction with an Enquire object.
@@ -54,27 +66,21 @@ namespace Xapian {
  *  which uses an incompatible format).
  */
 class XAPIAN_VISIBILITY_DEFAULT Database {
+	/// @internal Implementation behind check() static methods.
+	static size_t check_(const std::string * path_ptr, int fd, int opts,
+			     std::ostream *out);
+
+	/// Internal helper behind public compact() methods.
+	void compact_(const std::string * output_ptr,
+		      int fd,
+		      unsigned flags,
+		      int block_size,
+		      Xapian::Compactor * compactor) const;
+
     public:
 	class Internal;
 	/// @private @internal Reference counted internals.
-	std::vector<Xapian::Internal::RefCntPtr<Internal> > internal;
-
-	/** @private @internal Get a document from the database, but doesn't
-	 *  need to check if it exists.
-	 *
-	 *  This method returns a Xapian::Document object which provides the
-	 *  information about a document.  If the document doesn't exist,
-	 *  either a NULL pointer may be returned, or the returned object will
-	 *  throw DocNotFoundError when you try to access it.
-	 *
-	 *  The caller should delete the returned object when it has finished
-	 *  with it.
-	 *
-	 *  @param did   The document id of the document to retrieve.
-	 *
-	 *  @return      Pointer to Document::Internal object.
-	 */
-	Document::Internal * get_document_lazily(Xapian::docid did) const;
+	std::vector<Xapian::Internal::intrusive_ptr<Internal> > internal;
 
 	/** Add an existing database (or group of databases) to those
 	 *  accessed by this object.
@@ -83,6 +89,11 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 */
 	void add_database(const Database & database);
 
+	/** Return number of shards in this Database object. */
+	size_t size() const {
+	    return internal.size();
+	}
+
 	/** Create a Database with no databases in.
 	 */
 	Database();
@@ -91,8 +102,22 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  backend to use.
 	 *
 	 * @param path directory that the database is stored in.
+	 * @param flags  Bitwise-or of Xapian::DB_* constants.
 	 */
-	explicit Database(const std::string &path);
+	explicit Database(const std::string &path, int flags = 0);
+
+	/** Open a single-file Database.
+	 *
+	 *  This method opens a single-file Database given a file descriptor
+	 *  open on it.  Xapian looks starting at the current file offset,
+	 *  allowing a single file database to be easily embedded within
+	 *  another file.
+	 *
+	 * @param fd  file descriptor for the file.  Xapian takes ownership of
+	 *            this and will close it when the database is closed.
+	 * @param flags  Bitwise-or of Xapian::DB_* constants.
+	 */
+	explicit Database(int fd, int flags = 0);
 
 	/** @private @internal Create a Database from its internals.
 	 */
@@ -105,20 +130,28 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 */
 	virtual ~Database();
 
-        /** Copying is allowed.  The internals are reference counted, so
+	/** Copying is allowed.  The internals are reference counted, so
 	 *  copying is cheap.
 	 *
 	 *  @param other	The object to copy.
 	 */
 	Database(const Database &other);
 
-        /** Assignment is allowed.  The internals are reference counted,
+	/** Assignment is allowed.  The internals are reference counted,
 	 *  so assignment is cheap.
 	 *
 	 *  @param other	The object to copy.
 	 */
 	void operator=(const Database &other);
 
+#ifdef XAPIAN_MOVE_SEMANTICS
+	/// Move constructor.
+	Database(Database&& o);
+
+	/// Move assignment operator.
+	Database& operator=(Database&& o);
+#endif
+
 	/** Re-open the database.
 	 *
 	 *  This re-opens the database(s) to the latest available version(s).
@@ -127,16 +160,25 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *
 	 *  Calling reopen() on a database which has been closed (with @a
 	 *  close()) will always raise a Xapian::DatabaseError.
+	 *
+	 *  @return	true if the database might have been reopened (if false
+	 *		is returned, the database definitely hasn't been
+	 *		reopened, which applications may find useful when
+	 *		caching results, etc).  In Xapian < 1.3.0, this method
+	 *		did not return a value.
 	 */
-	void reopen();
+	bool reopen();
 
 	/** Close the database.
 	 *
-	 *  This closes the database and releases all file handles held by the
-	 *  database.
+	 *  This closes the database and closes all its file handles.
 	 *
-	 *  This cannot be undone - in particular, calling reopen() after
-	 *  closing a database will not reopen it, but will instead throw a
+	 *  For a WritableDatabase, if a transaction is active it will be
+	 *  aborted, while if no transaction is active commit() will be
+	 *  implicitly called.  Also the write lock is released.
+	 *
+	 *  Closing a database cannot be undone - in particular, calling
+	 *  reopen() after close() will not reopen it, but will instead throw a
 	 *  Xapian::DatabaseError exception.
 	 *
 	 *  Calling close() again on a database which has already been closed
@@ -158,6 +200,8 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  object associated with a Database, when in many cases they are
 	 *  working on data which has already been loaded and so they are able
 	 *  to just behave correctly.
+	 *
+	 *  This method was added in Xapian 1.1.0.
 	 */
 	virtual void close();
 
@@ -178,7 +222,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/** Corresponding end iterator to postlist_begin().
 	 */
-	PostingIterator postlist_end(const std::string &) const {
+	PostingIterator XAPIAN_NOTHROW(postlist_end(const std::string &) const) {
 	    return PostingIterator();
 	}
 
@@ -191,7 +235,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/** Corresponding end iterator to termlist_begin().
 	 */
-	TermIterator termlist_end(Xapian::docid) const {
+	TermIterator XAPIAN_NOTHROW(termlist_end(Xapian::docid) const) {
 	    return TermIterator();
 	}
 
@@ -205,36 +249,20 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/** Corresponding end iterator to positionlist_begin().
 	 */
-	PositionIterator positionlist_end(Xapian::docid, const std::string &) const {
+	PositionIterator XAPIAN_NOTHROW(positionlist_end(Xapian::docid, const std::string &) const) {
 	    return PositionIterator();
 	}
 
-	/** An iterator which runs across all terms in the database.
-	 */
-	TermIterator allterms_begin() const;
-
-	/** Corresponding end iterator to allterms_begin().
-	 */
-	TermIterator allterms_end() const {
-	    return TermIterator();
-	}
-
 	/** An iterator which runs across all terms with a given prefix.
 	 *
-	 *  This is functionally similar to getting an iterator with
-	 *  allterms_begin() and then calling skip_to(prefix) on that iterator
-	 *  to move to the start of the prefix, but is more convenient (because
-	 *  it detects the end of the prefixed terms), and may be more
-	 *  efficient than simply calling skip_to() after opening the iterator,
-	 *  particularly for remote databases.
-	 *
-	 *  @param prefix The prefix to restrict the returned terms to.
+	 *  @param prefix The prefix to restrict the returned terms to (default:
+	 *		  iterate all terms)
 	 */
-	TermIterator allterms_begin(const std::string & prefix) const;
+	TermIterator allterms_begin(const std::string & prefix = std::string()) const;
 
 	/** Corresponding end iterator to allterms_begin(prefix).
 	 */
-	TermIterator allterms_end(const std::string &) const {
+	TermIterator XAPIAN_NOTHROW(allterms_end(const std::string & = std::string()) const) {
 	    return TermIterator();
 	}
 
@@ -247,6 +275,12 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/// Get the average length of the documents in the database.
 	Xapian::doclength get_avlength() const;
 
+	/** Get the total length of all the documents in the database.
+	 *
+	 *  Added in Xapian 1.4.5.
+	 */
+	Xapian::totallength get_total_length() const;
+
 	/// Get the number of documents in the database indexed by a given term.
 	Xapian::doccount get_termfreq(const std::string & tname) const;
 
@@ -277,9 +311,6 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  stored in the slot.
 	 *
 	 *  @param slot The value slot to examine.
-	 *
-	 *  @exception UnimplementedError The frequency of the value isn't
-	 *  available for this database type.
 	 */
 	Xapian::doccount get_value_freq(Xapian::valueno slot) const;
 
@@ -288,9 +319,6 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  If there are no values stored in the given value slot, this will
 	 *  return an empty string.
 	 *
-	 *  If the lower bound isn't available for the given database type,
-	 *  this will return the lowest possible bound - the empty string.
-	 *
 	 *  @param slot The value slot to examine.
 	 */
 	std::string get_value_lower_bound(Xapian::valueno slot) const;
@@ -301,9 +329,6 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  return an empty string.
 	 *
 	 *  @param slot The value slot to examine.
-	 *
-	 *  @exception UnimplementedError The upper bound of the values isn't
-	 *  available for this database type.
 	 */
 	std::string get_value_upper_bound(Xapian::valueno slot) const;
 
@@ -323,13 +348,16 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	ValueIterator valuestream_begin(Xapian::valueno slot) const;
 
 	/// Return end iterator corresponding to valuestream_begin().
-	ValueIteratorEnd_ valuestream_end(Xapian::valueno) const {
-	    return ValueIteratorEnd_();
+	ValueIterator XAPIAN_NOTHROW(valuestream_end(Xapian::valueno) const) {
+	    return ValueIterator();
 	}
 
 	/// Get the length of a document.
 	Xapian::termcount get_doclength(Xapian::docid did) const;
 
+	/// Get the number of unique terms in document.
+	Xapian::termcount get_unique_terms(Xapian::docid did) const;
+
 	/** Send a "keep-alive" to remote databases to stop them timing out.
 	 *
 	 *  Has no effect on non-remote databases.
@@ -353,6 +381,25 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 */
 	Xapian::Document get_document(Xapian::docid did) const;
 
+	/** Get a document from the database, given its document id.
+	 *
+	 *  This method returns a Xapian::Document object which provides the
+	 *  information about a document.
+	 *
+	 *  @param did   The document id of the document to retrieve.
+	 *  @param flags Zero or more flags bitwise-or-ed together (currently
+	 *		 only Xapian::DOC_ASSUME_VALID is supported).
+	 *
+	 *  @return      A Xapian::Document object containing the document data
+	 *
+	 *  @exception Xapian::DocNotFoundError      The document specified
+	 *		could not be found in the database.
+	 *
+	 *  @exception Xapian::InvalidArgumentError  did was 0, which is not
+	 *		a valid document id.
+	 */
+	Xapian::Document get_document(Xapian::docid did, unsigned flags) const;
+
 	/** Suggest a spelling correction.
 	 *
 	 *  @param word			The potentially misspelled word.
@@ -374,7 +421,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	Xapian::TermIterator spellings_begin() const;
 
 	/// Corresponding end iterator to spellings_begin().
-	Xapian::TermIterator spellings_end() const {
+	Xapian::TermIterator XAPIAN_NOTHROW(spellings_end() const) {
 	    return Xapian::TermIterator();
 	}
 
@@ -385,7 +432,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	Xapian::TermIterator synonyms_begin(const std::string &term) const;
 
 	/// Corresponding end iterator to synonyms_begin(term).
-	Xapian::TermIterator synonyms_end(const std::string &) const {
+	Xapian::TermIterator XAPIAN_NOTHROW(synonyms_end(const std::string &) const) {
 	    return Xapian::TermIterator();
 	}
 
@@ -397,14 +444,14 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	Xapian::TermIterator synonym_keys_begin(const std::string &prefix = std::string()) const;
 
 	/// Corresponding end iterator to synonym_keys_begin(prefix).
-	Xapian::TermIterator synonym_keys_end(const std::string & = std::string()) const {
+	Xapian::TermIterator XAPIAN_NOTHROW(synonym_keys_end(const std::string & = std::string()) const) {
 	    return Xapian::TermIterator();
 	}
 
 	/** Get the user-specified metadata associated with a given key.
 	 *
 	 *  User-specified metadata allows you to store arbitrary information
-	 *  in the form of (key,tag) pairs.  See @a
+	 *  in the form of (key, value) pairs.  See @a
 	 *  WritableDatabase::set_metadata() for more information.
 	 *
 	 *  When invoked on a Xapian::Database object representing multiple
@@ -448,7 +495,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	Xapian::TermIterator metadata_keys_begin(const std::string &prefix = std::string()) const;
 
 	/// Corresponding end iterator to metadata_keys_begin().
-	Xapian::TermIterator metadata_keys_end(const std::string & = std::string()) const {
+	Xapian::TermIterator XAPIAN_NOTHROW(metadata_keys_end(const std::string & = std::string()) const) {
 	    return Xapian::TermIterator();
 	}
 
@@ -467,6 +514,262 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  contain the UUIDs of all the sub-databases.
 	 */
 	std::string get_uuid() const;
+
+	/** Test if this database is currently locked for writing.
+	 *
+	 *  If the underlying object is actually a WritableDatabase, always
+	 *  returns true.
+	 *
+	 *  Otherwise tests if there's a writer holding the lock (or if
+	 *  we can't test for a lock without taking it on the current platform,
+	 *  throw Xapian::UnimplementedError).  If there's an error while
+	 *  trying to test the lock, throws Xapian::DatabaseLockError.
+	 *
+	 *  For multi-databases, this tests each sub-database and returns
+	 *  true if any of them are locked.
+	 */
+	bool locked() const;
+
+	/** Get the revision of the database.
+	 *
+	 *  The revision is an unsigned integer which increases with each
+	 *  commit.
+	 *
+	 *  The database must have exactly one sub-database, which must be of
+	 *  type chert or glass.  Otherwise an exception will be thrown.
+	 *
+	 *  Experimental - see
+	 *  https://xapian.org/docs/deprecation#experimental-features
+	 */
+	Xapian::rev get_revision() const;
+
+	/** Check the integrity of a database or database table.
+	 *
+	 *  @param path	Path to database or table
+	 *  @param opts	Options to use for check
+	 *  @param out	std::ostream to write output to (NULL for no output)
+	 */
+	static size_t check(const std::string & path, int opts = 0,
+			    std::ostream *out = NULL) {
+	    return check_(&path, 0, opts, out);
+	}
+
+	/** Check the integrity of a single file database.
+	 *
+	 *  @param fd   file descriptor for the database.  The current file
+	 *              offset is used, allowing checking a single file
+	 *              database which is embedded within another file.  Xapian
+	 *              takes ownership of the file descriptor and will close
+	 *              it before returning.
+	 *  @param opts	Options to use for check
+	 *  @param out	std::ostream to write output to (NULL for no output)
+	 */
+	static size_t check(int fd, int opts = 0, std::ostream *out = NULL) {
+	    return check_(NULL, fd, opts, out);
+	}
+
+	/** Produce a compact version of this database.
+	 *
+	 *  New 1.3.4.  Various methods of the Compactor class were deprecated
+	 *  in 1.3.4.
+	 *
+	 *  @param output Path to write the compact version to.
+	 *		  This can be the same as an input if that input is a
+	 *		  stub database (in which case the database(s) listed
+	 *		  in the stub will be compacted to a new database and
+	 *		  then the stub will be atomically updated to point to
+	 *		  this new database).
+	 *
+	 *  @param flags Any of the following combined using bitwise-or (| in
+	 *		 C++):
+	 *   - Xapian::DBCOMPACT_NO_RENUMBER By default the document ids will
+	 *		be renumbered the output - currently by applying the
+	 *		same offset to all the document ids in a particular
+	 *		source database.  If this flag is specified, then this
+	 *		renumbering doesn't happen, but all the document ids
+	 *		must be unique over all source databases.  Currently
+	 *		the ranges of document ids in each source must not
+	 *		overlap either, though this restriction may be removed
+	 *		in the future.
+	 *   - Xapian::DBCOMPACT_MULTIPASS
+	 *		If merging more than 3 databases, merge the postlists
+	 *		in multiple passes, which is generally faster but
+	 *		requires more disk space for temporary files.
+	 *   - Xapian::DBCOMPACT_SINGLE_FILE
+	 *		Produce a single-file database (only supported for
+	 *		glass currently).
+	 *   - At most one of:
+	 *     - Xapian::Compactor::STANDARD - Don't split items unnecessarily.
+	 *     - Xapian::Compactor::FULL     - Split items whenever it saves
+	 *       space (the default).
+	 *     - Xapian::Compactor::FULLER   - Allow oversize items to save
+	 *       more space (not recommended if you ever plan to update the
+	 *       compacted database).
+	 *
+	 *  @param block_size This specifies the block size (in bytes) for
+	 *		to use for the output.  For glass, the block size must
+	 *		be a power of 2 between 2048 and 65536 (inclusive), and
+	 *		the default (also used if an invalid value is passed)
+	 *		is 8192 bytes.
+	 */
+	void compact(const std::string & output,
+		     unsigned flags = 0,
+		     int block_size = 0) {
+	    compact_(&output, 0, flags, block_size, NULL);
+	}
+
+	/** Produce a compact version of this database.
+	 *
+	 *  New 1.3.4.  Various methods of the Compactor class were deprecated
+	 *  in 1.3.4.
+	 *
+	 *  This variant writes a single-file database to the specified file
+	 *  descriptor.  Only the glass backend supports such databases, so
+	 *  this form is only supported for this backend.
+	 *
+	 *  @param fd   File descriptor to write the compact version to.  The
+	 *		descriptor needs to be readable and writable (open with
+	 *		O_RDWR) and seekable.  The current file offset is used,
+	 *		allowing compacting to a single file database embedded
+	 *		within another file.  Xapian takes ownership of the
+	 *		file descriptor and will close it before returning.
+	 *
+	 *  @param flags Any of the following combined using bitwise-or (| in
+	 *		C++):
+	 *   - Xapian::DBCOMPACT_NO_RENUMBER By default the document ids will
+	 *		be renumbered the output - currently by applying the
+	 *		same offset to all the document ids in a particular
+	 *		source database.  If this flag is specified, then this
+	 *		renumbering doesn't happen, but all the document ids
+	 *		must be unique over all source databases.  Currently
+	 *		the ranges of document ids in each source must not
+	 *		overlap either, though this restriction may be removed
+	 *		in the future.
+	 *   - Xapian::DBCOMPACT_MULTIPASS
+	 *		If merging more than 3 databases, merge the postlists
+	 *		in multiple passes, which is generally faster but
+	 *		requires more disk space for temporary files.
+	 *   - Xapian::DBCOMPACT_SINGLE_FILE
+	 *		Produce a single-file database (only supported for
+	 *		glass currently) - this flag is implied in this form
+	 *		and need not be specified explicitly.
+	 *
+	 *  @param block_size This specifies the block size (in bytes) for
+	 *		to use for the output.  For glass, the block size must
+	 *		be a power of 2 between 2048 and 65536 (inclusive), and
+	 *		the default (also used if an invalid value is passed)
+	 *		is 8192 bytes.
+	 */
+	void compact(int fd,
+		     unsigned flags = 0,
+		     int block_size = 0) {
+	    compact_(NULL, fd, flags, block_size, NULL);
+	}
+
+	/** Produce a compact version of this database.
+	 *
+	 *  New 1.3.4.  Various methods of the Compactor class were deprecated
+	 *  in 1.3.4.
+	 *
+	 *  The @a compactor functor allows handling progress output and
+	 *  specifying how user metadata is merged.
+	 *
+	 *  @param output Path to write the compact version to.
+	 *		  This can be the same as an input if that input is a
+	 *		  stub database (in which case the database(s) listed
+	 *		  in the stub will be compacted to a new database and
+	 *		  then the stub will be atomically updated to point to
+	 *		  this new database).
+	 *
+	 *  @param flags Any of the following combined using bitwise-or (| in
+	 *		 C++):
+	 *   - Xapian::DBCOMPACT_NO_RENUMBER By default the document ids will
+	 *		be renumbered the output - currently by applying the
+	 *		same offset to all the document ids in a particular
+	 *		source database.  If this flag is specified, then this
+	 *		renumbering doesn't happen, but all the document ids
+	 *		must be unique over all source databases.  Currently
+	 *		the ranges of document ids in each source must not
+	 *		overlap either, though this restriction may be removed
+	 *		in the future.
+	 *   - Xapian::DBCOMPACT_MULTIPASS
+	 *		If merging more than 3 databases, merge the postlists
+	 *		in multiple passes, which is generally faster but
+	 *		requires more disk space for temporary files.
+	 *   - Xapian::DBCOMPACT_SINGLE_FILE
+	 *		Produce a single-file database (only supported for
+	 *		glass currently).
+	 *
+	 *  @param block_size This specifies the block size (in bytes) for
+	 *		to use for the output.  For glass, the block size must
+	 *		be a power of 2 between 2048 and 65536 (inclusive), and
+	 *		the default (also used if an invalid value is passed)
+	 *		is 8192 bytes.
+	 *
+	 *  @param compactor Functor
+	 */
+	void compact(const std::string & output,
+		     unsigned flags,
+		     int block_size,
+		     Xapian::Compactor & compactor)
+	{
+	    compact_(&output, 0, flags, block_size, &compactor);
+	}
+
+	/** Produce a compact version of this database.
+	 *
+	 *  New 1.3.4.  Various methods of the Compactor class were deprecated
+	 *  in 1.3.4.
+	 *
+	 *  The @a compactor functor allows handling progress output and
+	 *  specifying how user metadata is merged.
+	 *
+	 *  This variant writes a single-file database to the specified file
+	 *  descriptor.  Only the glass backend supports such databases, so
+	 *  this form is only supported for this backend.
+	 *
+	 *  @param fd   File descriptor to write the compact version to.  The
+	 *		descriptor needs to be readable and writable (open with
+	 *		O_RDWR) and seekable.  The current file offset is used,
+	 *		allowing compacting to a single file database embedded
+	 *		within another file.  Xapian takes ownership of the
+	 *		file descriptor and will close it before returning.
+	 *
+	 *  @param flags Any of the following combined using bitwise-or (| in
+	 *		 C++):
+	 *   - Xapian::DBCOMPACT_NO_RENUMBER By default the document ids will
+	 *		be renumbered the output - currently by applying the
+	 *		same offset to all the document ids in a particular
+	 *		source database.  If this flag is specified, then this
+	 *		renumbering doesn't happen, but all the document ids
+	 *		must be unique over all source databases.  Currently
+	 *		the ranges of document ids in each source must not
+	 *		overlap either, though this restriction may be removed
+	 *		in the future.
+	 *   - Xapian::DBCOMPACT_MULTIPASS
+	 *		If merging more than 3 databases, merge the postlists
+	 *		in multiple passes, which is generally faster but
+	 *		requires more disk space for temporary files.
+	 *   - Xapian::DBCOMPACT_SINGLE_FILE
+	 *		Produce a single-file database (only supported for
+	 *		glass currently) - this flag is implied in this form
+	 *		and need not be specified explicitly.
+	 *
+	 *  @param block_size This specifies the block size (in bytes) for
+	 *		to use for the output.  For glass, the block size must
+	 *		be a power of 2 between 2048 and 65536 (inclusive), and
+	 *		the default (also used if an invalid value is passed)
+	 *		is 8192 bytes.
+	 *
+	 *  @param compactor Functor
+	 */
+	void compact(int fd,
+		     unsigned flags,
+		     int block_size,
+		     Xapian::Compactor & compactor)
+	{
+	    compact_(NULL, fd, flags, block_size, &compactor);
+	}
 };
 
 /** This class provides read/write access to a database.
@@ -475,13 +778,22 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
     public:
 	/** Destroy this handle on the database.
 	 *
-	 *  If there are no copies of this object remaining, the database
-	 *  will be closed.  If there are any transactions in progress
-	 *  these will be aborted as if cancel_transaction had been called.
+	 *  If no other handles to this database remain, the database will be
+	 *  closed.
+	 *
+	 *  If a transaction is active cancel_transaction() will be implicitly
+	 *  called; if no transaction is active commit() will be implicitly
+	 *  called, but any exception will be swallowed (because throwing
+	 *  exceptions in C++ destructors is problematic).  If you aren't using
+	 *  transactions and want to know about any failure to commit changes,
+	 *  call commit() explicitly before the destructor gets called.
 	 */
 	virtual ~WritableDatabase();
 
-	/** Create an empty WritableDatabase.
+	/** Create a WritableDatabase with no subdatabases.
+	 *
+	 *  The created object isn't very useful in this state - it's intended
+	 *  as a placeholder value.
 	 */
 	WritableDatabase();
 
@@ -493,34 +805,50 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  exist (but only the leaf directory, not recursively).
 	 *
 	 * @param path directory that the database is stored in.
-	 * @param action one of:
+	 * @param flags one of:
 	 *  - Xapian::DB_CREATE_OR_OPEN open for read/write; create if no db
-	 *    exists
+	 *    exists (the default if flags isn't specified)
 	 *  - Xapian::DB_CREATE create new database; fail if db exists
 	 *  - Xapian::DB_CREATE_OR_OVERWRITE overwrite existing db; create if
 	 *    none exists
 	 *  - Xapian::DB_OPEN open for read/write; fail if no db exists
 	 *
+	 *  Additionally, the following flags can be combined with action
+	 *  using bitwise-or (| in C++):
+	 *
+	 *   - Xapian::DB_NO_SYNC don't call fsync() or similar
+	 *   - Xapian::DB_DANGEROUS don't be crash-safe, no concurrent readers
+	 *   - Xapian::DB_RETRY_LOCK to wait to get a write lock
+	 *
+	 *  @param block_size If a new database is created, this specifies
+	 *		      the block size (in bytes) for backends which
+	 *		      have such a concept.  For chert and glass, the
+	 *		      block size must be a power of 2 between 2048 and
+	 *		      65536 (inclusive), and the default (also used if
+	 *		      an invalid value is passed) is 8192 bytes.
+	 *
 	 *  @exception Xapian::DatabaseCorruptError will be thrown if the
 	 *             database is in a corrupt state.
 	 *
 	 *  @exception Xapian::DatabaseLockError will be thrown if a lock
 	 *	       couldn't be acquired on the database.
 	 */
-	WritableDatabase(const std::string &path, int action);
+	explicit WritableDatabase(const std::string &path,
+				  int flags = 0,
+				  int block_size = 0);
 
 	/** @private @internal Create an WritableDatabase given its internals.
 	 */
 	explicit WritableDatabase(Database::Internal *internal);
 
-        /** Copying is allowed.  The internals are reference counted, so
+	/** Copying is allowed.  The internals are reference counted, so
 	 *  copying is cheap.
 	 *
 	 *  @param other	The object to copy.
 	 */
 	WritableDatabase(const WritableDatabase &other);
 
-        /** Assignment is allowed.  The internals are reference counted,
+	/** Assignment is allowed.  The internals are reference counted,
 	 *  so assignment is cheap.
 	 *
 	 *  Note that only an WritableDatabase may be assigned to an
@@ -531,6 +859,17 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 */
 	void operator=(const WritableDatabase &other);
 
+#ifdef XAPIAN_MOVE_SEMANTICS
+	/// Move constructor.
+	WritableDatabase(WritableDatabase&& o) : Database(std::move(o)) {}
+
+	/// Move assignment operator.
+	WritableDatabase& operator=(WritableDatabase&& o) {
+	    Database::operator=(std::move(o));
+	    return *this;
+	}
+#endif
+
 	/** Commit any pending modifications made to the database.
 	 *
 	 *  For efficiency reasons, when performing multiple updates to a
@@ -573,10 +912,9 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 
 	/** Pre-1.1.0 name for commit().
 	 *
-	 *  Use commit() instead in new code.  This alias may be deprecated in
-	 *  the future.
+	 *  Use commit() instead.
 	 */
-	void flush() { commit(); }
+	XAPIAN_DEPRECATED(void flush()) { commit(); }
 
 	/** Begin a transaction.
 	 *
@@ -624,7 +962,7 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *             called at an invalid time, such as when a transaction
 	 *             is already in progress.
 	 */
-	void begin_transaction(bool flushed=true);
+	void begin_transaction(bool flushed = true);
 
 	/** Complete the transaction currently in progress.
 	 *
@@ -871,11 +1209,11 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  existing item of metadata, just set its value to the empty string.
 	 *
 	 *  User-specified metadata allows you to store arbitrary information
-	 *  in the form of (key,tag) pairs.
+	 *  in the form of (key, value) pairs.
 	 *
 	 *  There's no hard limit on the number of metadata items, or the size
 	 *  of the metadata values.  Metadata keys have a limited length, which
-	 *  depends on the backend.  We recommend limiting them to 200 bytes.
+	 *  depend on the backend.  We recommend limiting them to 200 bytes.
 	 *  Empty keys are not valid, and specifying one will cause an
 	 *  exception.
 	 *
@@ -890,9 +1228,9 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *  associated with terms, documents, or postings by encoding the
 	 *  termname and/or document id into the metadata key.
 	 *
-	 *  @param key   The key of the metadata item to set.
+	 *  @param key       The key of the metadata item to set.
 	 *
-	 *  @param value The value of the metadata item to set.
+	 *  @param metadata  The value of the metadata item to set.
 	 *
 	 *  @exception Xapian::DatabaseError will be thrown if a problem occurs
 	 *             while writing to the database.
@@ -907,21 +1245,12 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 *             database backend in use doesn't support user-specified
 	 *             metadata.
 	 */
-	void set_metadata(const std::string & key, const std::string & value);
+	void set_metadata(const std::string & key, const std::string & metadata);
 
 	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
-/** Open for read/write; create if no db exists. */
-const int DB_CREATE_OR_OPEN = 1;
-/** Create a new database; fail if db exists. */
-const int DB_CREATE = 2;
-/** Overwrite existing db; create if none exists. */
-const int DB_CREATE_OR_OVERWRITE = 3;
-/** Open for read/write; fail if no db exists. */
-const int DB_OPEN = 4;
-
 }
 
 #endif /* XAPIAN_INCLUDED_DATABASE_H */
