@@ -1,7 +1,8 @@
 /** @file keymaker.h
  * @brief Build key strings for MSet ordering or collapsing.
  */
-/* Copyright (C) 2007,2009 Olly Betts
+/* Copyright (C) 2007,2009,2011,2013,2014,2015,2016 Olly Betts
+ * Copyright (C) 2010 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,10 +22,14 @@
 #ifndef XAPIAN_INCLUDED_KEYMAKER_H
 #define XAPIAN_INCLUDED_KEYMAKER_H
 
+#if !defined XAPIAN_IN_XAPIAN_H && !defined XAPIAN_LIB_BUILD
+# error Never use <xapian/keymaker.h> directly; include <xapian.h> instead.
+#endif
+
 #include <string>
 #include <vector>
 
-#include <xapian/deprecated.h>
+#include <xapian/intrusive_ptr.h>
 #include <xapian/types.h>
 #include <xapian/visibility.h>
 
@@ -33,8 +38,18 @@ namespace Xapian {
 class Document;
 
 /** Virtual base class for key making functors. */
-class XAPIAN_VISIBILITY_DEFAULT KeyMaker {
+class XAPIAN_VISIBILITY_DEFAULT KeyMaker
+    : public Xapian::Internal::opt_intrusive_base {
+    /// Don't allow assignment.
+    void operator=(const KeyMaker &);
+
+    /// Don't allow copying.
+    KeyMaker(const KeyMaker &);
+
   public:
+    /// Default constructor.
+    KeyMaker() { }
+
     /** Build a key string for a Document.
      *
      *  These keys can be used for sorting or collapsing matching documents.
@@ -45,6 +60,30 @@ class XAPIAN_VISIBILITY_DEFAULT KeyMaker {
 
     /** Virtual destructor, because we have virtual methods. */
     virtual ~KeyMaker();
+
+    /** Start reference counting this object.
+     *
+     *  You can hand ownership of a dynamically allocated KeyMaker
+     *  object to Xapian by calling release() and then passing the object to a
+     *  Xapian method.  Xapian will arrange to delete the object once it is no
+     *  longer required.
+     */
+    KeyMaker * release() {
+	opt_intrusive_base::release();
+	return this;
+    }
+
+    /** Start reference counting this object.
+     *
+     *  You can hand ownership of a dynamically allocated KeyMaker
+     *  object to Xapian by calling release() and then passing the object to a
+     *  Xapian method.  Xapian will arrange to delete the object once it is no
+     *  longer required.
+     */
+    const KeyMaker * release() const {
+	opt_intrusive_base::release();
+	return this;
+    }
 };
 
 /** KeyMaker subclass which combines several values.
@@ -63,66 +102,47 @@ class XAPIAN_VISIBILITY_DEFAULT KeyMaker {
  *  Other than this, it isn't useful to set @a reverse for collapsing.
  */
 class XAPIAN_VISIBILITY_DEFAULT MultiValueKeyMaker : public KeyMaker {
-    std::vector<std::pair<Xapian::valueno, bool> > slots;
+    struct KeySpec {
+	Xapian::valueno slot;
+	bool reverse;
+	std::string defvalue;
+	KeySpec(Xapian::valueno slot_, bool reverse_,
+		const std::string & defvalue_)
+		: slot(slot_), reverse(reverse_), defvalue(defvalue_)
+	{}
+    };
+    std::vector<KeySpec> slots;
 
   public:
     MultiValueKeyMaker() { }
 
-    template <class Iterator>
+    /** Construct a MultiValueKeyMaker from a pair of iterators.
+     *
+     *  The iterators must be a begin/end pair returning Xapian::valueno (or
+     *  a compatible type) when dereferenced.
+     */
+    template<class Iterator>
     MultiValueKeyMaker(Iterator begin, Iterator end) {
 	while (begin != end) add_value(*begin++);
     }
 
     virtual std::string operator()(const Xapian::Document & doc) const;
 
-    void add_value(Xapian::valueno slot, bool reverse = false) {
-	slots.push_back(std::make_pair(slot, reverse));
-    }
-};
-
-/** Virtual base class for sorter functor. */
-class XAPIAN_VISIBILITY_DEFAULT XAPIAN_DEPRECATED_CLASS Sorter : public KeyMaker { };
-
-/** Sorter subclass which sorts by a several values.
- *
- *  Results are ordered by the first value.  In the event of a tie, the
- *  second is used.  If this is the same for both, the third is used, and
- *  so on.
- *
- *  @deprecated This class is deprecated - you should migrate to using
- *  MultiValueKeyMaker instead.  Note that MultiValueSorter::add() becomes
- *  MultiValueKeyMaker::add_value(), but the sense of the direction flag
- *  is reversed (to be consistent with Enquire::set_sort_by_value()), so:
- *
- *    MultiValueSorter sorter;
- *    // Primary ordering is forwards on value 4.
- *    sorter.add(4);
- *    // Secondary ordering is reverse on value 5.
- *    sorter.add(5, false);
- *
- *  becomes:
- *
- *    MultiValueKeyMaker sorter;
- *    // Primary ordering is forwards on value 4.
- *    sorter.add_value(4);
- *    // Secondary ordering is reverse on value 5.
- *    sorter.add_value(5, true);
- */
-class XAPIAN_VISIBILITY_DEFAULT XAPIAN_DEPRECATED_CLASS MultiValueSorter : public Sorter {
-    std::vector<std::pair<Xapian::valueno, bool> > slots;
-
-  public:
-    MultiValueSorter() { }
-
-    template <class Iterator>
-    MultiValueSorter(Iterator begin, Iterator end) {
-	while (begin != end) add(*begin++);
-    }
-
-    virtual std::string operator()(const Xapian::Document & doc) const;
-
-    void add(Xapian::valueno slot, bool forward = true) {
-	slots.push_back(std::make_pair(slot, forward));
+    /** Add a value slot to the list to build a key from.
+     *
+     *  @param slot	The value slot to add
+     *  @param reverse	Adjust values from this slot to reverse their sort
+     *			order (default: false)
+     *	@param defvalue Value to use for documents which don't have a value
+     *			set in this slot (default: empty).  This can be used
+     *			to make such documents sort after all others by
+     *			passing <code>get_value_upper_bound(slot) + "x"</code>
+     *			- this is guaranteed to be greater than any value in
+     *			this slot.
+     */
+    void add_value(Xapian::valueno slot, bool reverse = false,
+		   const std::string & defvalue = std::string()) {
+	slots.push_back(KeySpec(slot, reverse, defvalue));
     }
 };
 
