@@ -29,12 +29,12 @@
 #warning The C++ ABI version of compiler you are using does not exactly match
 #warning that of the compiler used to build the library. If linking fails
 #warning due to missing symbols, this is probably the reason why.
-#warning The Xapian library was built with clang++ 8.0.1 (tags/RELEASE_801/final)
+#warning The Xapian library was built with clang++ 10.0.1 
 #else
 #error The C++ ABI version of compiler you are using does not match
 #error that of the compiler used to build the library. The versions
 #error must match or your program will not work correctly.
-#error The Xapian library was built with clang++ 8.0.1 (tags/RELEASE_801/final)
+#error The Xapian library was built with clang++ 10.0.1 
 #endif
 #endif
 
@@ -50,7 +50,7 @@
 #define XAPIAN_ENABLE_VISIBILITY
 
 /// The version of Xapian as a C string literal.
-#define XAPIAN_VERSION "1.4.15"
+#define XAPIAN_VERSION "1.4.16"
 
 /** The major component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 1
@@ -65,7 +65,7 @@
 /** The revision component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 14
  */
-#define XAPIAN_REVISION 15
+#define XAPIAN_REVISION 16
 
 /// Base (signed) type for Xapian::docid and related types.
 #define XAPIAN_DOCID_BASE_TYPE int
